@@ -3,6 +3,7 @@
 //! how transactions are included in blocks, and optimize those blocks.
 //!
 use {
+    crate::banking_stage::BatchedTransactionCostDetails,
     solana_measure::measure::Measure,
     solana_runtime::{
         bank::Bank,
@@ -128,6 +129,10 @@ impl QosService {
                             self.metrics.retried_txs_per_block_limit_count.fetch_add(1, Ordering::Relaxed);
                             Err(TransactionError::WouldExceedMaxBlockCostLimit)
                         }
+                        CostTrackerError::WouldExceedVoteMaxLimit => {
+                            self.metrics.retried_txs_per_vote_limit_count.fetch_add(1, Ordering::Relaxed);
+                            Err(TransactionError::WouldExceedMaxVoteCostLimit)
+                        }
                         CostTrackerError::WouldExceedAccountMaxLimit => {
                             self.metrics.retried_txs_per_account_limit_count.fetch_add(1, Ordering::Relaxed);
                             Err(TransactionError::WouldExceedMaxAccountCostLimit)
@@ -143,6 +148,36 @@ impl QosService {
         select_results
     }
 
+    pub fn accumulate_estimated_transaction_costs(
+        &self,
+        cost_details: &BatchedTransactionCostDetails,
+    ) {
+        self.metrics
+            .estimated_signature_cu
+            .fetch_add(cost_details.batched_signature_cost, Ordering::Relaxed);
+        self.metrics
+            .estimated_write_lock_cu
+            .fetch_add(cost_details.batched_write_lock_cost, Ordering::Relaxed);
+        self.metrics
+            .estimated_data_bytes_cu
+            .fetch_add(cost_details.batched_data_bytes_cost, Ordering::Relaxed);
+        self.metrics
+            .estimated_execute_cu
+            .fetch_add(cost_details.batched_execute_cost, Ordering::Relaxed);
+    }
+
+    pub fn accumulate_actual_execute_cu(&self, units: u64) {
+        self.metrics
+            .actual_execute_cu
+            .fetch_add(units, Ordering::Relaxed);
+    }
+
+    pub fn accumulate_actual_execute_time(&self, micro_sec: u64) {
+        self.metrics
+            .actual_execute_time_us
+            .fetch_add(micro_sec, Ordering::Relaxed);
+    }
+
     fn reporting_loop(
         running_flag: Arc<AtomicBool>,
         metrics: Arc<QosServiceMetrics>,
@@ -163,7 +198,26 @@ struct QosServiceMetrics {
     cost_tracking_time: AtomicU64,
     selected_txs_count: AtomicU64,
     retried_txs_per_block_limit_count: AtomicU64,
+    retried_txs_per_vote_limit_count: AtomicU64,
     retried_txs_per_account_limit_count: AtomicU64,
+
+    // accumulated estimated signature Compute Unites to be packed into block
+    estimated_signature_cu: AtomicU64,
+
+    // accumulated estimated write locks Compute Units to be packed into block
+    estimated_write_lock_cu: AtomicU64,
+
+    // accumulated estimated instructino data Compute Units to be packed into block
+    estimated_data_bytes_cu: AtomicU64,
+
+    // accumulated estimated program Compute Units to be packed into block
+    estimated_execute_cu: AtomicU64,
+
+    // accumulated actual program Compute Units that have been packed into block
+    actual_execute_cu: AtomicU64,
+
+    // accumulated actual program execute micro-sec that have been packed into block
+    actual_execute_time_us: AtomicU64,
 }
 
 impl QosServiceMetrics {
@@ -197,12 +251,48 @@ impl QosServiceMetrics {
                         .swap(0, Ordering::Relaxed) as i64,
                     i64
                 ),
+                (
+                    "retried_txs_per_vote_limit_count",
+                    self.retried_txs_per_vote_limit_count
+                        .swap(0, Ordering::Relaxed) as i64,
+                    i64
+                ),
                 (
                     "retried_txs_per_account_limit_count",
                     self.retried_txs_per_account_limit_count
                         .swap(0, Ordering::Relaxed) as i64,
                     i64
                 ),
+                (
+                    "estimated_signature_cu",
+                    self.estimated_signature_cu.swap(0, Ordering::Relaxed) as i64,
+                    i64
+                ),
+                (
+                    "estimated_write_lock_cu",
+                    self.estimated_write_lock_cu.swap(0, Ordering::Relaxed) as i64,
+                    i64
+                ),
+                (
+                    "estimated_data_bytes_cu",
+                    self.estimated_data_bytes_cu.swap(0, Ordering::Relaxed) as i64,
+                    i64
+                ),
+                (
+                    "estimated_execute_cu",
+                    self.estimated_execute_cu.swap(0, Ordering::Relaxed) as i64,
+                    i64
+                ),
+                (
+                    "actual_execute_cu",
+                    self.actual_execute_cu.swap(0, Ordering::Relaxed) as i64,
+                    i64
+                ),
+                (
+                    "actual_execute_time_us",
+                    self.actual_execute_time_us.swap(0, Ordering::Relaxed) as i64,
+                    i64
+                ),
             );
         }
     }
@@ -292,6 +382,7 @@ mod tests {
             .unwrap()
             .calculate_cost(&transfer_tx)
             .sum();
+        let vote_tx_cost = cost_model.read().unwrap().calculate_cost(&vote_tx).sum();
 
         // make a vec of txs
         let txs = vec![transfer_tx.clone(), vote_tx.clone(), transfer_tx, vote_tx];
@@ -299,19 +390,19 @@ mod tests {
         let qos_service = QosService::new(cost_model);
         let txs_costs = qos_service.compute_transaction_costs(txs.iter());
 
-        // set cost tracker limit to fit 1 transfer tx, vote tx bypasses limit check
-        let cost_limit = transfer_tx_cost;
+        // set cost tracker limit to fit 1 transfer tx and 1 vote tx
+        let cost_limit = transfer_tx_cost + vote_tx_cost;
         bank.write_cost_tracker()
             .unwrap()
-            .set_limits(cost_limit, cost_limit);
+            .set_limits(cost_limit, cost_limit, cost_limit);
         let results = qos_service.select_transactions_per_cost(txs.iter(), txs_costs.iter(), &bank);
 
-        // verify that first transfer tx and all votes are allowed
+        // verify that first transfer tx and first votes are allowed
         assert_eq!(results.len(), txs.len());
         assert!(results[0].is_ok());
         assert!(results[1].is_ok());
         assert!(results[2].is_err());
-        assert!(results[3].is_ok());
+        assert!(results[3].is_err());
     }
 
     #[test]
