@@ -49,6 +49,9 @@ use {
         snapshot_package::{
             AccountsPackageReceiver, AccountsPackageSender, PendingSnapshotPackage,
         },
+        transaction_cost_metrics_sender::{
+            TransactionCostMetricsSender, TransactionCostMetricsService,
+        },
         vote_sender_types::ReplayVoteSender,
     },
     solana_sdk::{clock::Slot, pubkey::Pubkey, signature::Keypair},
@@ -76,6 +79,7 @@ pub struct Tvu {
     cost_update_service: CostUpdateService,
     voting_service: VotingService,
     drop_bank_service: DropBankService,
+    transaction_cost_metrics_service: TransactionCostMetricsService,
 }
 
 pub struct Sockets {
@@ -313,6 +317,15 @@ impl Tvu {
         );
 
         let (drop_bank_sender, drop_bank_receiver) = channel();
+
+        let (tx_cost_metrics_sender, tx_cost_metrics_receiver) = unbounded();
+        let transaction_cost_metrics_sender = Some(TransactionCostMetricsSender::new(
+            cost_model.clone(),
+            tx_cost_metrics_sender,
+        ));
+        let transaction_cost_metrics_service =
+            TransactionCostMetricsService::new(tx_cost_metrics_receiver);
+
         let drop_bank_service = DropBankService::new(drop_bank_receiver);
 
         let replay_stage = ReplayStage::new(
@@ -336,6 +349,7 @@ impl Tvu {
             voting_sender,
             drop_bank_sender,
             block_metadata_notifier,
+            transaction_cost_metrics_sender,
         );
 
         let ledger_cleanup_service = tvu_config.max_ledger_shreds.map(|max_ledger_shreds| {
@@ -370,6 +384,7 @@ impl Tvu {
             cost_update_service,
             voting_service,
             drop_bank_service,
+            transaction_cost_metrics_service,
         }
     }
 
@@ -386,6 +401,7 @@ impl Tvu {
         self.cost_update_service.join()?;
         self.voting_service.join()?;
         self.drop_bank_service.join()?;
+        self.transaction_cost_metrics_service.join()?;
         Ok(())
     }
 }
