@@ -74,8 +74,10 @@
 #![allow(clippy::integer_arithmetic)]
 
 extern crate rand;
-use rand::{thread_rng, Rng};
-use std::collections::{HashMap, VecDeque};
+use {
+    rand::{thread_rng, Rng},
+    std::collections::{HashMap, VecDeque},
+};
 
 #[derive(Clone, Default, Debug, Hash, Eq, PartialEq)]
 pub struct Fork {
